"""
The `spiegel_scraper.py` module orchestrates the scraping and processing of Spiegel articles
between specified start and end dates. It handles the coordination between fetching archives and processing
the retrieved data.
"""

import asyncio
import logging
from asyncio import AbstractEventLoop
from concurrent.futures import ThreadPoolExecutor
from datetime import datetime, date, timedelta
from pathlib import Path

from src.spiegel_archive_fetcher import SpiegelArchiveFetcher
from src.processor import Processor

from typing import Any


class SpiegelScraper:
    """
    A class used to represent the scraper for Spiegel articles within a specified date range.

    Attributes
    ----------
    start_date : date
        The start date for the scraping process.
    end_date : date
        The end date for the scraping process.
    exclusions : list[str]
        A list of exclusion criteria (article types) for filtering articles. Given by urls.
    output_path : Path
        The path where the processed data will be saved.
    targets : list[str]
        A list of target criteria for filtering articles. Can be articles, comments or both.
    exclusion_count : int
        Counter for the number of articles excluded based on criteria.
    executor : ThreadPoolExecutor
        Executor to run blocking tasks in separate threads.
    archive_fetcher : SpiegelArchiveFetcher
        Instance of the SpiegelArchiveFetcher class.
    processor : Processor
        Instance of the Processor class.

    Methods
    -------
    scrap_article_data()
        Orchestrates the downloading and processing of article data.
    _process_date(target_date: date, chunk_size: int = 50)
        Processes articles for a specific date in chunks.
    """

    def __init__(
        self,
        start_date: str,
        end_date: str,
        exclusions: list[str],
        output_path: str,
        targets: list[str],
    ) -> None:
        """
        Constructs the SpiegelScraper object.

        Parameters
        ----------
        start_date : str
            The start date for the scraping process in 'YYYY-MM-DD' format.
        end_date : str
            The end date for the scraping process in 'YYYY-MM-DD' format.
        exclusions : list[str]
            A list of exclusion criteria for filtering articles.
        output_path : str
            The path where the processed data will be saved.
        targets : list[str]
            A list of target criteria for filtering articles.
        """
        self.start_date: date = datetime.strptime(start_date, "%Y-%m-%d")
        self.end_date: date = datetime.strptime(end_date, "%Y-%m-%d")
        self.exclusions: list[str] = exclusions
        self.output_path: Path = Path(output_path)
        self.targets: list[str] = targets
        self.exclusion_count: int = 0
        self.executor: ThreadPoolExecutor = ThreadPoolExecutor(max_workers=10)
        self.archive_fetcher: SpiegelArchiveFetcher = SpiegelArchiveFetcher(
            self.executor
        )
        self.processor: Processor = Processor(
            self.exclusion_count,
            self.exclusions,
            self.targets,
            self.output_path,
            self.executor,
        )

    def scrap_article_data(self) -> None:
        """
        Orchestrates the scraping and processing of article data within the specified date range.
        This function iterates over each day in the date range, fetches the corresponding archive,
        and processes the articles asynchronously.

        Raises
        ------
        asyncio.exceptions.Our Loop Closed Exception
            If the event loop couldn't be closed properly.
        """
        timedelta_start_end: timedelta = self.end_date - self.start_date
        loop: AbstractEventLoop = asyncio.get_event_loop()

        for number_of_days in range(timedelta_start_end.days + 1):
            current_date: date = self.start_date + timedelta(days=number_of_days)
            loop.run_until_complete(self._process_date(current_date))
        logging.info("All dates processed.")
        logging.info(
            f"{self.exclusion_count} articles were excluded according to the specified exclusion criteria"
        )

    async def _process_date(self, target_date: date, chunk_size: int = 50) -> None:
        """
        Processes articles for a specific date.

        This method fetches the archive for the provided date and processes the articles
        in chunks (Chunk processing was found to be more efficient).

        Parameters
        ----------
        target_date : date
            The date for which articles need to be processed.
        chunk_size : int, optional
            The number of articles to process concurrently in a single batch (default is 50).

        Returns
        -------
        None
        """
        logging.info(f"Processing date: {target_date}")
        archive: list[dict[str, Any]] = await self.archive_fetcher.fetch_archive(
            target_date
        )
        await self.processor.process_archive(archive, target_date, chunk_size)
        logging.info(f"Finished date: {target_date}")
